import yaml
import os
from ADDU.logic import docker_snippets as ds
import docker
from pathlib import Path

current_file_path = Path(__file__).parent
config_path = current_file_path / ".." / "config" / "config.yaml"
config_path = config_path.resolve()
CONFIG = yaml.safe_load(open(config_path, "r"))
WORKSPACES_PATH = Path(CONFIG["addu-workspaces-path"]).expanduser()


class Workspace:
    def __init__(self, workspace_name=None, user=None, distro=None, base_image=None, editor=None):
        self.workspace_name = workspace_name
        self.workspace_path = WORKSPACES_PATH / workspace_name
        print(self.workspace_path)
        self.user = user
        self.distro = distro
        self.base_image = base_image
        self.image_name = f"{self.distro}:{self.workspace_name}-{self.user}"
        self.container_name = f"{self.workspace_name}-{self.user}"
        self.editor = editor

    def create_workspace(self):
        # Settings dirs
        os.makedirs(f"{self.workspace_path}/settings/java", exist_ok=True)
        os.makedirs(f"{self.workspace_path}/settings/config", exist_ok=True)
        os.makedirs(f"{self.workspace_path}/settings/local", exist_ok=True)
        os.makedirs(f"{self.workspace_path}/settings/cache", exist_ok=True)
        # Shared Volume dir
        os.makedirs(f"{self.workspace_path}/shared", exist_ok=True)
        # Dockerfile, config.yaml and startup.sh
        self.create_dockerfile()
        self.create_config_yaml()
        match self.editor:
            case "vscode":
                self.create_startup(command="code")
            case "pycharm-professional":
                self.create_startup(command="pycharm.sh")
            case "pycharm-community":
                self.create_startup(command="pycharm.sh")

    def create_config_yaml(self):
        with open(f"{self.workspace_path}/config.yaml", "w") as config:
            config.write(f"workspace_name: {self.workspace_name}\n")
            config.write(f"user: {self.user}\n")
            config.write(f"distro: {self.distro}\n")
            config.write(f"base_image: {self.base_image}\n")
            config.write(f"image_name: {self.image_name}\n")
            config.write(f"container_name: {self.container_name}\n")
            config.write(f"editor: {self.editor}\n")

    def create_dockerfile(self):
        with open(f"{self.workspace_path}/Dockerfile", "w") as dockerfile:
            lines = [
                f"FROM {self.base_image}",
                ds.install_packages(["python3-pip", "wget", "ranger", "git", "libxrender1", "libxtst6", "libxi6"]),
                ds.create_user(self.user),
                ds.source_ros_in_user(self.user, self.distro),
                ds.grafical_support(),
                ds.install_editor(self.editor)
            ]
            dockerfile.writelines(lines)

    def create_startup(self, command="bash"):
        with open(f"{self.workspace_path}/startup.sh", "w") as startup:
            lines = (
                f"docker run -it --privileged "
                f"--name {self.workspace_name} "
                f"--user {self.user} "
                f"--workdir /home/{self.user} "
                f"--env DISPLAY=$DISPLAY "
                f"--volume /dev:/dev "
                f"--volume {self.workspace_path}/shared:/home/{self.user}/shared "
                f"--volume {self.workspace_path}/settings/config:/home/{self.user}/.config "
                f"--volume {self.workspace_path}/settings/cache:/home/{self.user}/.cache "
                f"--volume {self.workspace_path}/settings/local:/home/{self.user}/.local "
                f"--volume {self.workspace_path}/settings/java:/home/{self.user}/.java "
                f"--network host "
                f"--rm "
                f"{self.image_name} "
                f"{command}"
            )
            startup.writelines(lines)

    def run_workspace(self):
        os.system(f"bash {self.workspace_path}/startup.sh")

    def build_image(self):
        client = docker.from_env()
        client.images.build(path=self.workspace_path, tag=self.image_name, rm=True, forcerm=True)


if __name__ == '__main__':
    w = Workspace("test", "test", "noetic", "ros:noetic", "pycharm-professional")



import time

from rich.console import Console
from rich.prompt import Prompt

from ADDU.cli.addu_panels import (option_panel, about_panel, workspace_creation_panel,
                                  list_workspaces, delete_workspace, run_workspace)
from ADDU.logic import workspace as ws


def addu_cli():
    console = Console()
    options = option_panel()
    console.print(options)
    options = ['about', 'create', 'list', 'run', 'delete', "exit"]
    option = Prompt.ask("Choose an option", choices=options)
    console.clear()
    match option:
        case 'about':
            console.print(about_panel())
            input("Enter to go back")
            console.clear()
            addu_cli()
        case 'create':
            ws_name, base_image, user, distro, editor = workspace_creation_panel(console)
            workspace = ws.Workspace(ws_name, user, distro, base_image, editor)
            console.print("[bold blue]Creating workspace...[bold blue]")
            workspace.create_workspace()
            console.print(f"[bold green]Workspace {ws_name} created![/bold green]")
            console.print("[bold blue]Building image...[bold blue]")
            workspace.build_image()
            console.print(f"[bold green]Image built![/bold green]")
            time.sleep(1)
            console.clear()
            addu_cli()
        case 'list':
            console.print(list_workspaces())
            input("Enter to go back")
            console.clear()
            addu_cli()
        case 'run':
            run_workspace(console)
            console.clear()
            addu_cli()
        case 'delete':
            delete_workspace(console)
            console.clear()
            addu_cli()
        case 'exit':
            console.print("[bold green]Goodbye![/bold green]")


if __name__ == '__main__':
    addu_cli()



def install_packages(packages_list):
    snippet = f"""
RUN apt-get update && \\
    apt-get install -y \\
"""
    for pkg in packages_list:
        if packages_list[-1] == pkg:
            snippet += f"    {pkg}"
        else:
            snippet += f"    {pkg} \\\n"
    return snippet


def create_user(name, uid=1000):
    snippet = f"""
ARG USER={name}
ARG UID={uid}
ARG GID=$UID
RUN groupadd --gid $GID $USER && \\
    useradd -m -s /bin/bash --uid $UID --gid $GID $USER && \\
    echo "$USER:$USER" | chpasswd && \\
    usermod -aG sudo $USER && \\
    echo "$USER ALL=(ALL) NOPASSWD: ALL" > /etc/sudoers.d/$USER 
RUN mkdir -p /home/$USER/shared
    """
    return snippet


def source_ros_in_user(user, ros_version):
    snippet = f"""
RUN echo "source /opt/ros/{ros_version}/setup.bash" >> /home/{user}/.bashrc
RUN echo "source /home/{user}/shared/ros_ws/install/setup.bash" >> /home/{user}/.bashrc
    """
    if ros_version == "noetic":
        snippet = f"""
RUN echo "source /opt/ros/{ros_version}/setup.bash" >> /home/{user}/.bashrc
RUN echo "source /home/{user}/shared/ros_ws/devel/setup.bash" >> /home/{user}/.bashrc
"""
    return snippet


def install_editor(editor):
    if editor == "vscode":
        snippet = f"""
RUN wget -O vscode.tar.gz https://update.code.visualstudio.com/latest/linux-x64/stable
RUN tar -xvf vscode.tar.gz -C /opt/
ENV PATH="/opt/VSCode-linux-x64:${{PATH}}"
    """
    if editor == "pycharm-professional":
        snippet = f"""
RUN wget -q "https://download.jetbrains.com/python/pycharm-professional-2023.3.3.tar.gz"
RUN tar -xvf pycharm-professional-2023.3.3.tar.gz -C /opt/
ENV PATH="/opt/pycharm-2023.3.3/bin:${{PATH}}"
"""
    if editor == "pycharm-community":
        snippet = f"""
RUN wget -q https://download.jetbrains.com/python/pycharm-community-2023.3.3.tar.gz
RUN tar -xvf pycharm-community-2023.3.3.tar.gz -C /opt/
ENV PATH="/opt/pycharm-community-2023.3.3/bin:${{PATH}}"
"""
    return snippet


def grafical_support():
    snippet = f"""
VOLUME /tmp/.X11-unix:/tmp/.X11-unix:rw
RUN apt-get install -y \\
    libx11-6 \\
    libxext-dev \\
    libxrender-dev \\
    libxtst6 \\
    x11-apps \\
    x11-utils \\
    libgl1-mesa-glx \\
    libgl1-mesa-dri \\
    libgtk2.0-0 \\
    libgtk-3-0\n
    """
    return snippet



import os
import shutil
import yaml

from rich.console import Console
from rich.panel import Panel
from rich.prompt import Prompt
from rich.text import Text
from rich.table import Table

from pathlib import Path

current_file_path = Path(__file__).parent
config_path = current_file_path / ".." / "config" / "config.yaml"
config_path = config_path.resolve()

CONFIG = yaml.load(open(config_path, "r"), Loader=yaml.FullLoader)
workspaces_path = Path(CONFIG["addu-workspaces-path"]).expanduser()
if not os.path.exists(workspaces_path):
    os.makedirs(workspaces_path)


def about_panel():
    titulo = Text("ADDU ", "bold")
    subtitle = "A Dumb Docker User"
    descripcion = Text.assemble(
        Text("ADDU is a creation and management tool of "),
        Text("ROS development environments.\n", "bold green"),
        Text("With ADDU you can run your favorite editor inside a container with all ROS dependencies installed.\n\n"),
        Text("\n\n Maintained by: Daiego43"),
    )
    descripcion.justify = "center"
    return Panel.fit(descripcion, title=titulo, border_style="blue", subtitle=subtitle, padding=(1, 2))


def option_panel():
    opciones = Text.assemble(
        Text("about  -  About ADDU\n", "grey"),
        Text("create -  Create a workspace\n", "blue"),
        Text("list   -  List workspaces\n", "bold yellow"),
        Text("run    -  Run a workspace\n", "bold green"),
        Text("delete -  Delete a workspace\n", "bold red"),
        Text("exit   -  Exit ADDU cli\n", "bold purple"),
    )
    return Panel.fit(opciones, title=":whale: ADDU :whale:", border_style="cyan", subtitle="A Dumb Docker User",
                     padding=(1, 2))


def workspace_creation_panel(console):
    config = CONFIG
    ws_name = Prompt.ask("[green]Workspace name[/green]")
    distro = Prompt.ask("[green]Choose a ROS distro[/green]", choices=config["ros-distros"].keys())
    console.print(f"[bold green]Available images for {distro}[/bold green]")
    for i, image in enumerate(config["ros-distros"][distro]):
        console.print(f" {i} - [cyan]{image}[/cyan]")
    image = Prompt.ask("[green]Choose an image[/green]",
                       choices=[str(i) for i in range(len(config["ros-distros"][distro]))])
    user = Prompt.ask("[green]Provide a username for the image[/green]")
    console.print(f"[bold green]Available editors[/bold green]")
    for i, editor in enumerate(config["supported-editors"]):
        console.print(f" {i} - [cyan]{editor}[/cyan]")
    editor = Prompt.ask("[green]Choose an editor[/green]",
                        choices=[str(i) for i in range(len(config["supported-editors"]))])
    editor = config["supported-editors"][int(editor)]
    image = config["ros-distros"][distro][int(image)]
    console.clear()
    return ws_name, image, user, distro, editor


def list_workspaces():
    title = Text("Workspaces", "bold")
    subtitle = Text("Workspaces", "bold")
    workspaces_path = Path(CONFIG["addu-workspaces-path"]).expanduser()
    workspaces = os.listdir(workspaces_path)
    table = Table()
    table.add_column("ID", style="cyan")
    table.add_column("Workspace", style="magenta")
    table.add_column("Distro", style="green")
    table.add_column("User", style="blue")
    table.add_column("Image name", style="red")
    if not workspaces:
        return Panel.fit("No workspaces found!", title=title, border_style="blue", subtitle=subtitle, padding=(1, 2))

    for i, ws in enumerate(workspaces):
        ws_config = yaml.load(open(f"{workspaces_path}/{ws}/config.yaml", "r"), Loader=yaml.FullLoader)
        table.add_row(str(i), ws, ws_config["distro"], ws_config["user"], ws_config["base_image"])

    return Panel.fit(table, title=title, border_style="blue", subtitle=subtitle, padding=(1, 2))


def delete_workspace(console):
    list_panel = list_workspaces()
    workspaces_path = Path(CONFIG["addu-workspaces-path"]).expanduser()
    workspaces = os.listdir(workspaces_path)
    console.print(list_panel)
    if not workspaces:
        return
    i = Prompt.ask("Enter the workspace id you want to delete", choices=[str(i) for i in range(len(workspaces))])
    path = workspaces_path / workspaces[int(i)]
    shutil.rmtree(path)


def run_workspace(console):
    list_panel = list_workspaces()
    workspaces_path = Path(CONFIG["addu-workspaces-path"]).expanduser()
    workspaces = os.listdir(workspaces_path)
    console.print(list_panel)
    if not workspaces:
        return
    i = Prompt.ask("Enter the workspace id you want to run", choices=[str(i) for i in range(len(workspaces))])
    ws = workspaces[int(i)]
    os.system(f"bash {workspaces_path}/{ws}/startup.sh")
